@@ -7,6 +7,7 @@ and needs to be converted to a Class.
 import logging
 import pprint
 import time
+from datetime import datetime
 from datetime import timedelta
 from distutils.version import StrictVersion
 from os import get_terminal_size
@@ -1264,6 +1265,7 @@ def list_alarms(speaker, action, args, soco_function, use_local_speaker_list):
             duration = "No Limit"
         details.append(
             [
+                alarm._alarm_id,  # Replace with .alarm_id with SoCo v0.22
                 alarm.zone.player_name,
                 time,
                 duration,
@@ -1276,6 +1278,7 @@ def list_alarms(speaker, action, args, soco_function, use_local_speaker_list):
             ]
         )
     headers = [
+        "Alarm ID",
         "Speaker",
         "Start Time",
         "Duration",
@@ -1292,6 +1295,182 @@ def list_alarms(speaker, action, args, soco_function, use_local_speaker_list):
     return True
 
 
+@one_parameter
+def remove_alarms(speaker, action, args, soco_function, use_local_speaker_list):
+
+    alarms = soco.alarms.get_alarms(speaker)
+
+    if args[0].lower() == "all":
+        for alarm in alarms:
+            logging.info("Removing alarm ID '{}'".format(alarm._alarm_id))
+            alarm.remove()
+        return True
+
+    alarm_ids_to_delete = args[0].split(",")
+    alarm_ids_to_delete = set(alarm_ids_to_delete)
+    logging.info("Attempting to delete alarm ID(s): {}".format(alarm_ids_to_delete))
+
+    alarm_ids = {alarm._alarm_id for alarm in alarms}
+    logging.info("Current alarm ID(s): {}".format(alarm_ids))
+
+    valid_alarm_ids_to_delete = alarm_ids.intersection(alarm_ids_to_delete)
+    logging.info("Valid alarm ID(s) to delete: {}".format(valid_alarm_ids_to_delete))
+
+    for alarm in alarms:
+        if alarm._alarm_id in valid_alarm_ids_to_delete:
+            logging.info("Deleting alarm ID: {}".format(alarm._alarm_id))
+            alarm.remove()
+
+    alarms_invalid = alarm_ids_to_delete.difference(valid_alarm_ids_to_delete)
+    if len(alarms_invalid) != 0:
+        print("Error: Alarm ID(s) {} not found".format(alarms_invalid))
+
+    return True
+
+
+@one_parameter
+def add_alarm(speaker, action, args, soco_function, use_local_speaker_list):
+    alarm_parameters = args[0].split(",")
+    if len(alarm_parameters) != 8:
+        error_and_exit("8 comma-separated parameters must be supplied")
+        return False
+
+    start_time = alarm_parameters[0]
+    try:
+        start_time = datetime.strptime(start_time, "%H:%M").time()
+    except ValueError:
+        error_and_exit("Invalid time format: {}".format(start_time))
+        return False
+
+    duration = alarm_parameters[1]
+    try:
+        duration = datetime.strptime(duration, "%H:%M").time()
+    except ValueError:
+        error_and_exit("Invalid time format: {}".format(duration))
+        return False
+
+    recurrence = alarm_parameters[2]
+    if not soco.alarms.is_valid_recurrence(recurrence):
+        error_and_exit("'{}' is not a valid recurrence string".format(recurrence))
+        return False
+
+    enabled = alarm_parameters[3].lower()
+    if enabled == "on":
+        enabled = True
+    elif enabled == "off":
+        enabled = False
+    else:
+        error_and_exit(
+            "Alarm must be enabled 'on' or 'off', not '{}'".format(alarm_parameters[3])
+        )
+        return False
+
+    uri = alarm_parameters[4]
+    if uri.lower() == "chime":
+        uri = None
+
+    play_mode = alarm_parameters[5].upper()
+    play_mode_options = [
+        "NORMAL",
+        "SHUFFLE_NOREPEAT",
+        "SHUFFLE",
+        "REPEAT_ALL",
+        "REPEAT_ONE",
+        "SHUFFLE_REPEAT_ONE",
+    ]
+    if play_mode not in play_mode_options:
+        error_and_exit(
+            "Play mode is '{}', should be one of:\n  {}".format(
+                alarm_parameters[5], play_mode_options
+            )
+        )
+        return False
+
+    volume = alarm_parameters[6]
+    try:
+        volume = int(volume)
+        if not 0 <= volume <= 100:
+            error_and_exit(
+                "Alarm volume must be between 0 and 100, not '{}'".format(
+                    alarm_parameters[6]
+                )
+            )
+            return False
+    except ValueError:
+        error_and_exit(
+            "Alarm volume must be an integer between 0 and 100, not '{}'".format(
+                alarm_parameters[6]
+            )
+        )
+        return False
+
+    include_linked = alarm_parameters[7].lower()
+    if include_linked == "on":
+        include_linked = True
+    elif include_linked == "off":
+        include_linked = False
+    else:
+        error_and_exit(
+            "Linked zones must be enabled 'on' or 'off', not '{}'".format(
+                alarm_parameters[7]
+            )
+        )
+        return False
+
+    alarm = soco.alarms.Alarm(
+        speaker,
+        start_time=start_time,
+        duration=duration,
+        recurrence=recurrence,
+        enabled=enabled,
+        program_uri=uri,
+        play_mode=play_mode,
+        volume=volume,
+        include_linked_zones=include_linked,
+    )
+    try:
+        alarm.save()
+    except soco.exceptions.SoCoUPnPException:
+        error_and_exit("Failed to create alarm")
+        return False
+
+    print("Alarm ID '{}' created".format(alarm._alarm_id))
+    return True
+
+
+@two_parameters
+def enable_alarms(speaker, action, args, soco_function, use_local_speaker_list):
+
+    if args[1].lower() == "on":
+        enabled = True
+    elif args[1].lower() == "off":
+        enabled = False
+    else:
+        error_and_exit("Second parameter must be 'on' or 'off'")
+        return False
+
+    alarms = soco.alarms.get_alarms(speaker)
+
+    alarm_ids = set(args[0].lower().split(","))
+    all_alarms = True if "all" in alarm_ids else False
+    if all_alarms:
+        alarm_ids.discard("all")
+
+    for alarm in alarms:
+        if all_alarms is True or alarm._alarm_id in alarm_ids:
+            logging.info(
+                "Setting alarm id '{}' to enabled = {}".format(alarm._alarm_id, enabled)
+            )
+            alarm.enabled = enabled
+            alarm.save()
+            alarm_ids.discard(alarm._alarm_id)
+
+    if len(alarm_ids) != 0:
+        print("Alarm IDs not found: {}".format(alarm_ids))
+
+    return True
+
+
 @zero_parameters
 def list_libraries(speaker, action, args, soco_function, use_local_speaker_list):
     shares = speaker.music_library.list_library_shares()
@@ -2093,7 +2272,8 @@ def wait_end_track(speaker, action, args, soco_function, use_local_speaker_list)
             event = sub.events.get(timeout=1.0)
             logging.info("Transport event received")
 
-            # The code below never works; retain for possible future use
+            # The code below didn't work; retain for possible future use
+            # Consider using a countdown?
             #
             # info = speaker.get_current_track_info()
             # position = convert_to_seconds(info["position"])
@@ -2454,4 +2634,10 @@ actions = {
     "playpause": SonosFunction(pauseplay, "", True),
     "available_actions": SonosFunction(available_actions, "", True),
     "wait_end_track": SonosFunction(wait_end_track, "", True),
+    "remove_alarms": SonosFunction(remove_alarms, "", False),
+    "remove_alarm": SonosFunction(remove_alarms, "", False),
+    "add_alarm": SonosFunction(add_alarm, "", False),
+    "create_alarm": SonosFunction(add_alarm, "", False),
+    "enable_alarm": SonosFunction(enable_alarms, "", False),
+    "enable_alarms": SonosFunction(enable_alarms, "", False),
 }
